@@ -72,16 +72,23 @@ int get_horiz_start() {
   return 30;
 }
 
-GdkPixmap* get_pixmap (char* filename)
+GdkPixmap* get_pixmap (const char* filename)
 {
   GdkPixmap* ret;
   GdkImlibImage *im;
   char* fullname = gnome_pixmap_file (filename);
 
+  if (fullname == NULL)
+    return NULL; 
+
   im = gdk_imlib_load_image (fullname);
-  gdk_imlib_render (im, im->rgb_width, im->rgb_height);
-  ret = gdk_imlib_copy_image (im);
-  gdk_imlib_destroy_image (im);
+  if (im != NULL) {
+    gdk_imlib_render (im, im->rgb_width, im->rgb_height);
+    ret = gdk_imlib_copy_image (im);
+    gdk_imlib_destroy_image (im);
+  } else {
+    ret = NULL;
+  } 
   g_free (fullname);
 
   return ret;
@@ -98,8 +105,10 @@ void load_pixmaps(GtkWidget* app, GdkCardDeckOptions deck_options)
 void free_pixmaps() 
 {
   gtk_object_destroy (card_deck);
-  gdk_pixmap_unref (slot_pixmap);
-  gdk_pixmap_unref (default_background_pixmap);
+  if (slot_pixmap != NULL)
+    gdk_pixmap_unref (slot_pixmap);
+  if (default_background_pixmap != NULL)
+    gdk_pixmap_unref (default_background_pixmap);
 }
 
 void add_card(GList** card_list, hcard_type temp_card) {
