@@ -1,5 +1,5 @@
 /*
- * written by J. Marcin Gorycki <mgo@olicom.dk>
+ * written by J. Marcin Gorycki <marcin.gorycki@intel.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,8 +23,12 @@
 
 Field::	Field()
 {
-	bg = NULL;
+	bg = 0;
+	gtk_widget_push_visual (gdk_imlib_get_visual ());
+	gtk_widget_push_colormap (gdk_imlib_get_colormap ());
 	w = gnome_canvas_new();
+	gtk_widget_pop_colormap ();
+	gtk_widget_pop_visual ();
 }
 
 void 
@@ -33,7 +37,7 @@ Field::show()
 	gtk_widget_realize(w);
 
 	gtk_widget_show(w);
-	updateSize(NULL);
+	updateSize(0);
 }
 
 void
@@ -55,7 +59,7 @@ Field::updateSize(GdkImlibImage * bgImage)
   			"width", (double) COLUMNS * BLOCK_SIZE,
   			"height", (double) LINES * BLOCK_SIZE,
   			"anchor", GTK_ANCHOR_NW,
-  			NULL);
+  			0);
 		else
 			bg = gnome_canvas_item_new(
   			gnome_canvas_root(GNOME_CANVAS(w)),
@@ -67,5 +71,5 @@ Field::updateSize(GdkImlibImage * bgImage)
 				"fill_color", "black",
 				"outline_color", "black",
 				"width_units", 1.0,
-  			NULL);
+  			0);
 }
