@@ -19,6 +19,7 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <libgnomeui/gnome-window-icon.h>
 #include <time.h>
 
 #define APPNAME "gnotravex"
@@ -206,7 +207,7 @@ int main (int argc, char **argv){
   textdomain(PACKAGE);
 
   gnome_init_with_popt_table(APPNAME, VERSION, argc, argv, options, 0, NULL);
-
+  gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnotravex/gnome-gnotravex.png");
   client = gnome_master_client();
   gtk_object_ref(GTK_OBJECT(client));
   gtk_object_sink(GTK_OBJECT(client));
