@@ -32,10 +32,25 @@
 
 #define SCROLL_WIDTH 10000
 
-#define TAPE_FONT "-bitstream-courier-bold-r-*-*-25-*-*-*-*-*-*-*"
+#define STATE 0
+#define READ 1
+#define WRITE 2
+#define MOVE 3
+#define NEW_STATE 4
+#define ENTRIES_MAX 5
 
 /* A few globals. */
+char *tape_font[] = 
+{
+	"-bitstream-courier-bold-r-*-*-25-*-*-*-*-*-*-*",
+	"-adobe-courier-bold-r-normal-*-24-*-*-*-m-*-iso8859-1",
+	"-adobe-courier-bold-r-normal-*-25-*-*-*-m-*-iso8859-1",
+	"fixed",
+	NULL
+};
+
 static char *progname;
+static char *prog_message = NULL;
 static char tape_string[1024] = "";
 static char states_fname[1024] = "";
 
@@ -57,8 +72,22 @@ static GtkWidget *step;
 static GtkWidget *statusline;
 static GtkWidget *helpline;
 
+static GtkWidget *save;
+static GtkWidget *edit_save = NULL;
+
+void power_do (void);
 void power_callback(GtkWidget *power_button, gpointer data);
 void state_clist_load_states(void);
+void state_clist_select_state(turing *t);
+void save_call(GtkWidget *w, gpointer data);
+
+void set_save_sens(gboolean saves)
+{
+	gtk_widget_set_sensitive (save, saves);
+	
+	if (edit_save)
+		gtk_widget_set_sensitive (edit_save, saves);
+}
 
 void set_toolbar_sens(gboolean powers, gboolean stops,
 											gboolean plays, gboolean steps)
@@ -91,14 +120,26 @@ void set_tape(char *str)
 	free(tmp);
 }
 
+void view_comment (void)
+{
+	GtkWidget *w;
+	char *mess;
+	
+	if (prog_message && *prog_message && *prog_message != '\n')
+		mess = prog_message;
+	else
+		mess = _("No comment for this program.");
+	
+	w = gnome_message_box_new(mess, GNOME_MESSAGE_BOX_INFO,
+														GNOME_STOCK_BUTTON_OK, NULL);
+	GTK_WINDOW(w)->position = GTK_WIN_POS_MOUSE;
+	gtk_widget_show(w);
+}
+
 void set_states_fname(char *str)
 {
 	char *c;
 	
-	strncpy(states_fname, str, 1024);
-	if (turing_fread_states(tm, states_fname))
-		*states_fname = 0;
-	
 	if ((*states_fname != 0) && (*tape_string != 0))
 		power_callback(power, NULL);
 	else
@@ -110,29 +151,65 @@ void set_states_fname(char *str)
 		c = states_fname;
 
 	state_clist_load_states();
+	state_clist_select_state(tm);
   gtk_label_set(GTK_LABEL(helpline), c);
 }
 
+void states_fname_examples_callback (GtkWidget *ok_button, gpointer data)
+{
+	GtkFileSelection *fsel;
+	
+	fsel = GTK_FILE_SELECTION (data);
+	gtk_file_selection_set_filename (fsel, GTURING_EXAMPLES_DIR);
+}
+
 void states_fname_callback(GtkWidget *ok_button, gpointer data)
 {
-	char *fname, *message;
-	GtkWidget *w;
+	gboolean action_save;
+	char *fname, *comment;
+	GtkWidget *w, *text;
 	
 	fname = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(dialog)));
-	gtk_widget_destroy(dialog);
-	dialog = NULL;
+	
+	action_save = (gboolean) data;
+	if (action_save)
+	{
+		text = gtk_object_get_data (GTK_OBJECT (dialog), "text");
+		comment = gtk_editable_get_chars (GTK_EDITABLE (text), 0, -1);
+		if (turing_fwrite_states (tm->statehead, fname, comment))
+			*states_fname = 0; /*error*/
+		else
+			strncpy(states_fname, fname, 1024);
+		
+		if (prog_message)
+			free (prog_message);
+		
+		prog_message = strdup (comment);
+	}
+	else
+	{
+		if (turing_fread_states(tm, fname))
+			*states_fname = 0; /*error*/
+		else
+		{
+			strncpy(states_fname, fname, 1024);
+		
+			if (prog_message)
+				free(prog_message);
+			
+			prog_message = turing_fread_comments(fname);
+			view_comment ();
+			
+			set_save_sens (TRUE);
+		}
+	}
 	
 	set_states_fname(fname);
-	message = turing_fread_comments(fname);
-	w = gnome_message_box_new(message, GNOME_MESSAGE_BOX_INFO,
-														GNOME_STOCK_BUTTON_OK, NULL);
-	GTK_WINDOW(w)->position = GTK_WIN_POS_MOUSE;
-	gtk_widget_show(w);
-	
-	free(message);
 	g_free(fname);
 
 	gnome_config_set_string("/gTuring/Options/program", states_fname);
+	gtk_widget_destroy(dialog);
+	dialog = NULL;
 }
 
 void tape_string_callback(GtkWidget *ok_button, gpointer data)
@@ -144,7 +221,7 @@ void tape_string_callback(GtkWidget *ok_button, gpointer data)
 	gtk_widget_destroy(dialog);
 	dialog = NULL;
 	
-	if ((*states_fname != 0) && (*tape_string != 0))
+	if (tm->statehead && (*tape_string != 0))
 		power_callback(power, NULL);
 	else
 		set_toolbar_sens(FALSE, FALSE, FALSE, FALSE);
@@ -173,9 +250,164 @@ void cancel_callback(GtkWidget *widget, gpointer data)
 	dialog = NULL;
 }
 
+void states_view_edit_set_clicked_callback (GtkWidget *widget, gpointer data)
+{
+	GtkWidget **entries, *message;
+	gchar *c[ENTRIES_MAX];
+	turing_state state, *s;
+	int i;
+	
+	entries = gtk_object_get_data (GTK_OBJECT (widget), "entries");
+	
+	for (i = 0; i < ENTRIES_MAX; i++)
+		c[i] = gtk_entry_get_text (GTK_ENTRY (entries[i]));
+
+	/* FIXME: states should be able to be greater than 9. */
+	if (!(*c[STATE] >= '0' && *c[STATE] <= '9') ||
+			(*c[MOVE] != 'l' && *c[MOVE] != 'r') ||
+			!(*c[NEW_STATE] >= '0' && *c[NEW_STATE] <= '9'))
+	{
+		gtk_widget_grab_focus (entries[0]);
+		message = gnome_message_box_new (_("States must be numbers. Move can only be 'r' or 'l'."),
+																		 GNOME_MESSAGE_BOX_ERROR,
+																		 GNOME_STOCK_BUTTON_OK,
+																		 NULL);
+		gtk_widget_show (message);
+		return;
+	}
+	
+	state.no = atoi (c[STATE]);
+	state.read = *c[READ];
+	state.write = *c[WRITE];
+	state.move = *c[MOVE];
+	state.new = atoi (c[NEW_STATE]);
+	state.next = NULL;
+	
+	turing_set_state (tm, state);
+	
+	gtk_clist_freeze (GTK_CLIST (state_clist));
+	state_clist_load_states ();
+	gtk_clist_thaw (GTK_CLIST (state_clist));
+	
+	for (s = tm->statehead; s; s = s->next)
+		if ((s->no == state.no) && (s->read == state.read))
+			break;
+	
+	if (s)
+		for (i = 0; ; i++)
+			if (gtk_clist_get_row_data (GTK_CLIST(state_clist), i) == s)
+	    {
+				gtk_clist_select_row(GTK_CLIST(state_clist), i, 0);
+				break;
+			}
+
+	if (tm->tapehead)
+		power_do ();
+
+	set_save_sens (TRUE);
+	
+	gtk_widget_grab_focus (entries[0]);
+}
+
+void states_view_entry_grab_focus_callback (GtkWidget *widget, gpointer data)
+{
+	GtkWidget **entries;
+	int i;
+	
+	entries = gtk_object_get_data (GTK_OBJECT (widget), "entries");
+	
+	for (i = 0; i < ENTRIES_MAX; i++)
+		gtk_editable_select_region (GTK_EDITABLE (entries[i]), 0, 0);
+	
+	gtk_editable_select_region (GTK_EDITABLE (widget), 0, -1);
+}
+
+void states_view_edit_callback (GtkWidget *widget, gpointer data)
+{
+	GtkWidget *w;
+	GtkWidget *table, *label, *frame, *button;
+	GtkWidget **entries;
+	GtkWidget *message;
+	gint i;
+	gchar *labels[] = { N_("State"), N_("Read"), N_("Write"), N_("Move"), N_("New State"), NULL };
+	
+	message = gnome_message_box_new (_("Editing is still alpha. Come back later or hack."),
+																	 GNOME_MESSAGE_BOX_WARNING,
+																	 GNOME_STOCK_BUTTON_OK,
+																	 NULL);
+	gtk_widget_show (message);
+	
+	gtk_widget_set_sensitive (widget, FALSE);
+	
+	w = GTK_WIDGET (data);
+	entries = g_new (GtkWidget *, ENTRIES_MAX);
+	gtk_object_set_data (GTK_OBJECT (state_clist), "entries", entries);
+	
+	frame = gtk_frame_new (_("Edit Or Create Row"));
+	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(w)->vbox), frame, FALSE, FALSE, 0);
+	table = gtk_table_new (2, 2 * ENTRIES_MAX + 2, FALSE);
+	gtk_container_add (GTK_CONTAINER (frame), table);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 8);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 8);
+
+	edit_save = button = gnome_pixmap_button (gnome_stock_pixmap_widget (w, GNOME_STOCK_PIXMAP_SAVE_AS),
+																						_("Save"));
+	gtk_signal_connect (GTK_OBJECT (button), "clicked", 
+											save_call, NULL);
+	gtk_widget_set_sensitive (edit_save, GTK_WIDGET_IS_SENSITIVE (save));
+	gtk_table_attach (GTK_TABLE (table), button, ENTRIES_MAX * 2 + 1, ENTRIES_MAX * 2 + 2, 1, 2, 0, 
+										GTK_FILL | GTK_EXPAND, 0, 0);
+	
+
+	button = gnome_pixmap_button (gnome_stock_pixmap_widget (w, GNOME_STOCK_PIXMAP_ADD),
+																_("Set"));
+	gtk_object_set_data (GTK_OBJECT (button), "entries", entries);
+	gtk_table_attach (GTK_TABLE (table), button, ENTRIES_MAX * 2, ENTRIES_MAX * 2 + 1, 1, 2, 0, 
+										GTK_FILL | GTK_EXPAND, 0, 0);
+	
+	for (i = 0; labels[i]; i++)
+	{
+#ifdef ENABLE_NLS
+		label = gtk_label_new (_(labels[i]));
+#else
+		label = gtk_label_new (labels[i]);
+#endif
+		entries[i] = gtk_entry_new_with_max_length (1);
+		gtk_object_set_data (GTK_OBJECT (entries[i]), "entries", entries);
+		gtk_widget_set_usize (entries[i], 20, 0);
+
+		gtk_signal_connect (GTK_OBJECT (entries[i]), "grab_focus", 
+												states_view_entry_grab_focus_callback, NULL);
+		if (i != 0)
+			gtk_signal_connect_object (GTK_OBJECT (entries[i - 1]), "activate", 
+																 gtk_widget_grab_focus, GTK_OBJECT (entries[i]));
+																			 
+		gtk_table_attach (GTK_TABLE (table), label, i * 2, i * 2 + 1, 0, 1, 0, 0, 0, 0);
+		gtk_table_attach (GTK_TABLE (table), entries[i], i * 2, i * 2 + 1, 1, 2, 0, 0, 0, 0);
+		gtk_table_attach (GTK_TABLE (table), gtk_vseparator_new (), 
+											i * 2 + 1, i * 2 + 2, 0, 2, 0, GTK_FILL | GTK_EXPAND, 0, 0);
+	}
+	gtk_signal_connect_object (GTK_OBJECT (entries[i - 1]), "activate", 
+														 gtk_widget_grab_focus, GTK_OBJECT (button));
+	gtk_signal_connect_object (GTK_OBJECT (button), "clicked", 
+														 gtk_widget_grab_focus, GTK_OBJECT (entries[0]));
+	gtk_signal_connect (GTK_OBJECT (button), "clicked", 
+											states_view_edit_set_clicked_callback, NULL);
+
+	gtk_widget_show_all (frame);
+}
+
 void states_view_close_callback(GtkWidget *w, gpointer data)
 {
+	GtkWidget **entries;
+	
+	entries = gtk_object_get_data (GTK_OBJECT (w), "entries");
+	if (entries)
+		g_free (entries);
+	
 	state_clist = NULL;
+	edit_save = NULL;
 	
 	gtk_widget_destroy(w);
 }
@@ -184,6 +416,7 @@ void state_clist_select_state(turing *t)
 {
 	gint i, tmp;
 	char buff[20];
+	GtkWidget **entries;
 
 	if (t->actualstate) {
 		snprintf(buff, 20, _("State: %d"), t->actualstate->no);
@@ -195,14 +428,15 @@ void state_clist_select_state(turing *t)
 		tmp = i = GPOINTER_TO_INT (gtk_object_get_user_data(GTK_OBJECT(state_clist)));
 		while (i--)
 			if (t->actualstate == 
-					(state *) gtk_clist_get_row_data(GTK_CLIST(state_clist), i))
+					(turing_state *) gtk_clist_get_row_data(GTK_CLIST(state_clist), i))
 				break;
 		
 		if (i >= 0) {
 			gtk_clist_select_row(GTK_CLIST(state_clist), i, 0);
 			/* bug: moveto corrupts the clist if all the rows appear in the list */
-			if (tmp > 11)
+/*			if (tmp > 11)*/
 				gtk_clist_moveto(GTK_CLIST(state_clist), (i - 5 < 5)? 0 : i - 5, 0, 0, 0);
+			
 		}
 	}
 }
@@ -217,7 +451,7 @@ int next_state(turing *t)
 	return ret;
 }
 
-void power_callback(GtkWidget *power_button, gpointer data)
+void power_do (void)
 {
 	stop_flag = 1;
 	tm->state = 0;
@@ -230,6 +464,11 @@ void power_callback(GtkWidget *power_button, gpointer data)
 	set_toolbar_sens(FALSE, FALSE, TRUE, TRUE);
 }
 
+void power_callback(GtkWidget *power_button, gpointer data)
+{
+	power_do ();
+}
+
 gint do_play(gpointer data)
 {
 	char *tmp;
@@ -310,18 +549,60 @@ void prompt(char *title, char *msg, GtkSignalFunc callback, char *def)
 	gtk_widget_show(dialog);
 }
 
+void save_call(GtkWidget *w, gpointer data) 
+{
+	GtkWidget *fsel;
+	GtkWidget *frame, *frame2, *scrolled, *text;
+	int i = 0;
+	char *txt;
+	
+	dialog = fsel = gtk_file_selection_new(_("Save gTuring Program File"));
+	
+	frame2 = gtk_frame_new (_("Comment"));
+	gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (fsel)->main_vbox), frame2, TRUE, TRUE, 0);
+	frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
+	gtk_container_set_border_width (GTK_CONTAINER (frame), 8);
+	gtk_container_add (GTK_CONTAINER (frame2), frame);
+	scrolled = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled), 
+																	GTK_POLICY_AUTOMATIC,
+																	GTK_POLICY_AUTOMATIC);
+	gtk_container_add (GTK_CONTAINER (frame), scrolled);
+	text = gtk_text_new (NULL, NULL);
+	gtk_text_set_editable (GTK_TEXT (text), TRUE);
+	if (prog_message)
+		gtk_editable_insert_text (GTK_EDITABLE (text), prog_message, strlen (prog_message), &i);
+	gtk_container_add (GTK_CONTAINER (scrolled), text);
+	
+	gtk_widget_show_all (frame2);
+
+	gtk_file_selection_set_filename (GTK_FILE_SELECTION(fsel), states_fname);
+	gtk_object_set_data (GTK_OBJECT (fsel), "text", text);
+	gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(fsel)->ok_button), "clicked",
+											GTK_SIGNAL_FUNC(states_fname_callback), (gpointer) TRUE);
+	gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(fsel)->cancel_button), "clicked", 
+											GTK_SIGNAL_FUNC(cancel_callback), NULL);
+	gtk_widget_show(fsel);
+}
+
 void open_call(GtkWidget *w, gpointer data) 
 {
 	GtkWidget *fsel;
+	GtkWidget *button;
 	        
 	dialog = fsel = gtk_file_selection_new(_("Open gTuring Program File"));
-	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fsel));
-	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(fsel)->ok_button),
-														"clicked", GTK_SIGNAL_FUNC(states_fname_callback),
-														GTK_OBJECT(fsel));
-	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(fsel)->cancel_button),
-														"clicked", GTK_SIGNAL_FUNC(cancel_callback),
-														GTK_OBJECT(fsel));
+	
+	button = gtk_button_new_with_label ("Examples");
+	gtk_container_add (GTK_CONTAINER (GTK_FILE_SELECTION (fsel)->button_area), button);
+	gtk_widget_show (button);
+	
+	gtk_signal_connect (GTK_OBJECT(button), "clicked", 
+											GTK_SIGNAL_FUNC(states_fname_examples_callback), fsel);
+	gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(fsel)->ok_button), "clicked",
+											GTK_SIGNAL_FUNC(states_fname_callback), (gpointer) FALSE);
+	gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(fsel)->cancel_button), "clicked", 
+											GTK_SIGNAL_FUNC(cancel_callback), NULL);
 	gtk_widget_show(fsel);
 }
 
@@ -334,7 +615,7 @@ void tape_call(GtkWidget *w, gpointer data)
 void state_clist_load_states(void)
 {
 	char *text[5];
-	state *s;
+	turing_state *s;
 	gint i;
 
 	if (state_clist) {
@@ -361,8 +642,25 @@ void state_clist_load_states(void)
 		
 		for (i = 0; i < 5; i++)
 			free(text[i]);
-		
-		state_clist_select_state(tm);
+	}
+}
+
+void state_clist_select_row_callback (GtkCList *clist, gint row, gint column, GdkEventButton *event,
+																			gpointer user_data)
+{
+	GtkWidget **entries;
+	
+	entries = gtk_object_get_data (GTK_OBJECT (clist), "entries");
+	if (entries)
+	{
+		int i;
+		gchar *txt;
+				
+		for (i = 0; i < ENTRIES_MAX; i++)
+		{
+			gtk_clist_get_text (GTK_CLIST (state_clist), row, i, &txt);
+			gtk_entry_set_text (GTK_ENTRY (entries[i]), txt);
+		}
 	}
 }
 
@@ -370,15 +668,20 @@ void view_states_call(GtkWidget *widget, gpointer data)
 {
 	char *text[5] = { N_("State"), N_("Read"), N_("Write"), N_("Move"), 
 		                N_("New State")	};
-	GtkWidget *w;
+	GtkWidget *w, *vbox;
 	GtkWidget *scrolled;
 	int i;
 	
 	if (state_clist)
 		return;
 	
-	w = gnome_dialog_new(_("Machine's states"), GNOME_STOCK_BUTTON_CLOSE, NULL);
-	gnome_dialog_button_connect_object(GNOME_DIALOG(w),	0,
+	w = gnome_dialog_new(_("Machine's states"), NULL);
+	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (w), "  Edit >>", GNOME_STOCK_PIXMAP_PROPERTIES);
+	gnome_dialog_append_button (GNOME_DIALOG (w), GNOME_STOCK_BUTTON_CLOSE);
+	gtk_window_set_policy (GTK_WINDOW (w), TRUE, TRUE, FALSE);
+	gnome_dialog_button_connect (GNOME_DIALOG(w),	0,
+															 GTK_SIGNAL_FUNC(states_view_edit_callback), w);
+	gnome_dialog_button_connect_object(GNOME_DIALOG(w),	1,
 																		 GTK_SIGNAL_FUNC(states_view_close_callback),
 																		 GTK_OBJECT(w));
 
@@ -391,15 +694,15 @@ void view_states_call(GtkWidget *widget, gpointer data)
 	
 	state_clist = gtk_clist_new_with_titles(5, text);
 	gtk_clist_set_selection_mode(GTK_CLIST(state_clist), GTK_SELECTION_SINGLE);
-/*	gtk_clist_set_policy(GTK_CLIST(state_clist), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC); */
 	gtk_clist_column_titles_passive(GTK_CLIST(state_clist));
 	for (i = 0; i < 5; i++)
 		gtk_clist_set_column_width(GTK_CLIST(state_clist), i, 60);
 	gtk_widget_set_usize(state_clist, 360, 200);
 	
 	state_clist_load_states();
-
-	/* Needed for the new clist api  -- Greg*/
+	state_clist_select_state(tm);
+	
+	gtk_signal_connect (GTK_OBJECT (state_clist), "select_row", state_clist_select_row_callback, NULL);
 
 	scrolled = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled), 
@@ -407,11 +710,16 @@ void view_states_call(GtkWidget *widget, gpointer data)
 					GTK_POLICY_AUTOMATIC);
 	gtk_container_add (GTK_CONTAINER (scrolled), state_clist);
 
-	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(w)->vbox), scrolled, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(w)->vbox), scrolled, TRUE, TRUE, 0);
   gtk_widget_show_all(GNOME_DIALOG(w)->vbox);
 	gtk_widget_show(w);
 }
 
+void view_comment_call(GtkWidget *w, gpointer data)
+{
+	view_comment ();
+}
+
 void exit_call(GtkWidget *w, gpointer data)
 {
 	gtk_main_quit();
@@ -450,7 +758,8 @@ void about_call(GtkWidget *w, gpointer data)
 
 GnomeUIInfo filemenu[] = {
 
-        GNOMEUIINFO_MENU_OPEN_ITEM(open_call, NULL),
+	GNOMEUIINFO_MENU_OPEN_ITEM(open_call, NULL),
+	GNOMEUIINFO_MENU_SAVE_AS_ITEM(save_call, NULL),
 
 	GNOMEUIINFO_SEPARATOR,
 
@@ -459,15 +768,22 @@ GnomeUIInfo filemenu[] = {
 	GNOMEUIINFO_END
 };
 
+GnomeUIInfo viewmenu[] = {
+  
+	GNOMEUIINFO_ITEM_NONE(N_("_Comment..."), N_("View the program's comment."),
+			      view_comment_call),
+	GNOMEUIINFO_ITEM_NONE(N_("_States..."),
+			      N_("Open a table with the machine's states."),
+			      view_states_call),
+	GNOMEUIINFO_END
+};
+
 GnomeUIInfo settingsmenu[] = {
   
 	GNOMEUIINFO_ITEM_NONE(N_("_Play Speed..."), N_("Set playing speed."),
 			      playspeed_call),
 	GNOMEUIINFO_ITEM_NONE(N_("_Tape..."), N_("Set the machine's tape."),
 			      tape_call),
-	GNOMEUIINFO_ITEM_NONE(N_("_View States"),
-			      N_("Open a table with the machine's states."),
-			      view_states_call),
 	GNOMEUIINFO_END
 };
 
@@ -480,6 +796,7 @@ GnomeUIInfo helpmenu[] = {
 GnomeUIInfo mainmenu[] = {
         GNOMEUIINFO_MENU_FILE_TREE(filemenu),
 	GNOMEUIINFO_MENU_SETTINGS_TREE(settingsmenu),
+	GNOMEUIINFO_MENU_VIEW_TREE(viewmenu),
 	GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
 	GNOMEUIINFO_END
 };
@@ -496,15 +813,34 @@ GnomeUIInfo toolbar[] = {
 	GNOMEUIINFO_END
 };
 
+char *try_font (void)
+{
+	GdkFont *font;
+	int i;
+	
+	i = 0;
+	font = NULL;
+	while (tape_font[i] && !font)
+		font = gdk_font_load (tape_font[i]);
+	
+	return tape_font[i];
+}
+
 void create_machine(void)
 {
 	GtkWidget *frame;
+	GtkRcStyle *style;
 	
 	frame = rootw;
 	tapelabel = gtk_label_new((*tape_string != 0)? tape_string : _("Welcome to gTuring."));
-	headlabel = gtk_label_new("^                   ");
-	headlabel->style = tapelabel->style = gtk_style_copy(tapelabel->style);
-	tapelabel->style->font = gdk_font_load(TAPE_FONT);
+	/* This string is supposed to have the same length as the welcoming string. */
+	headlabel = gtk_label_new(_("^                   "));
+	
+	style = gtk_rc_style_new ();
+	style->font_name = try_font ();
+	gtk_widget_modify_style (GTK_WIDGET (tapelabel), style);
+	gtk_widget_modify_style (GTK_WIDGET (headlabel), style);
+	
 	gtk_box_pack_start(GTK_BOX(rootw), tapelabel, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(rootw), headlabel, TRUE, TRUE, 0);
 }
@@ -566,6 +902,8 @@ void init_interface(int argc, char *argv[])
 	
 	app = gnome_app_new("gturing", "gturing");
 	gtk_window_set_wmclass(GTK_WINDOW(app), "gturing", "gturing");
+	gtk_signal_connect(GTK_OBJECT(app), "delete_event",
+			   GTK_SIGNAL_FUNC(exit_call), NULL);
 	gtk_widget_show(app);
 
 	rootw = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
@@ -573,11 +911,14 @@ void init_interface(int argc, char *argv[])
 	gnome_app_create_menus(GNOME_APP(app), mainmenu);
 	gnome_app_create_toolbar(GNOME_APP(app), toolbar);
 
+	save = filemenu[1].widget;
+	
 	power = toolbar[0].widget;
 	stop = toolbar[1].widget;
 	play = toolbar[2].widget;
 	step = toolbar[3].widget;
 
+	set_save_sens (FALSE);
 	set_toolbar_sens(FALSE, FALSE, FALSE, FALSE);
 	
 	create_machine();
@@ -588,7 +929,7 @@ void init_interface(int argc, char *argv[])
 
 void init_globals(void)
 {
-	tm = new_turing();
+	tm = turing_new();
 	
 	strncpy(tape_string, gnome_config_get_string("/gTuring/Options/tape="), 1024);
 	strncpy(states_fname, gnome_config_get_string("/gTuring/Options/program="), 1024);
@@ -638,8 +979,6 @@ int main (int argc, char *argv[])
 	parse_args(argc, argv);
 	gnome_init("gnomecard", VERSION, argc, argv);
 
-		chdir(GTURING_EXAMPLES_DIR);
-		
 	init_globals();
 	init_interface(argc, argv);
 	
